@@ -25,6 +25,5 @@ class TestLHC_LSA(TestCpymad, unittest.TestCase):
         self.model._cmd('option,-twiss_print')
 
 if __name__ == '__main__':
-    suite = unittest.TestLoader().loadTestsFromTestCase(TestLHC_LSA)
-    unittest.TextTestRunner(verbosity=1).run(suite)
+    unittest.main()
 
